﻿namespace ProjectGasolineTanker;
/// <summary>
/// Класс, отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawingGasolineTanker
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
   public EntityGasolineTanker? EntityGasolineTanker { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окнаы
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки бензовоза
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя координата прорисовки бензовоза
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки бензовоза
    /// </summary>
    private readonly int _drawingGasolineTankerWidth = 105;
    /// <summary>
    /// Высота прорисовки бензовоза
    /// </summary>
    private readonly int _drawingGasolineTankerHeight = 70;

    /// <summary>
    /// Инициализация всех свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="gasTank">Признак наличия бака под бензин</param>
    /// <param name="signalBeacon">Признак наличия сигнального маяка на кабине</param>
    public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool gasTank, bool signalBeacon)
    {
        EntityGasolineTanker = new EntityGasolineTanker();
        EntityGasolineTanker.Init(speed, weight, bodyColor, additionalColor, gasTank, signalBeacon);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns></returns>
    public bool SetPictureSize(int width, int height)
    {
        // Проверка "влезает" ли объект в размеры поля
        if (_drawingGasolineTankerWidth > width || _drawingGasolineTankerHeight > height)
        {
            return false;
        }
        _pictureWidth = width;
        _pictureHeight = height;

        if (_startPosX.HasValue || _startPosY.HasValue)
        {
            if (_drawingGasolineTankerWidth + _startPosX > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _drawingGasolineTankerWidth;
            }
            else if (_startPosX < 0)
            {
                _startPosX = 0;
            }
            if (_drawingGasolineTankerHeight + _startPosY > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _drawingGasolineTankerHeight;
            }
            else if (_startPosY < 0)
            {
                _startPosY = 0;
            }
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата х</param>
    /// <param name="y">Координата у</param>
    
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        if (x >= 0 && x + _drawingGasolineTankerWidth <= _pictureWidth && y >= 0 && y + _drawingGasolineTankerHeight <= _pictureHeight)
        {
        _startPosX = x;
        _startPosY = y; 
        
        }
        if (x < 0) 
        {
            _startPosX = 0;
        }
        if (y < 0) 
        {
            _startPosY = 0;
        }
        if (x + _drawingGasolineTankerWidth > _pictureWidth) 
        {
            _startPosX = _drawingGasolineTankerWidth - _pictureWidth;
        }
        if (y + _drawingGasolineTankerHeight > _pictureHeight) 
        {
            _startPosY = _drawingGasolineTankerHeight - _pictureHeight;
        }
    }
    /// <summary>
    /// Изменение направления
    /// </summary>
    /// <param name="direction">Направление</param>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityGasolineTanker == null || !_startPosX.HasValue || !_startPosY.HasValue)
        { 
            return false;

        }
        switch(direction)
        {
            // влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityGasolineTanker.Step > 0)
                {
                    _startPosX -= (int)EntityGasolineTanker.Step;
                }
                return true;
            // вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityGasolineTanker.Step > 0)
                {
                    _startPosY -= (int)EntityGasolineTanker.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityGasolineTanker.Step + _drawingGasolineTankerWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityGasolineTanker.Step;
                }
                return true;
            // вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityGasolineTanker.Step + _drawingGasolineTankerHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityGasolineTanker.Step;
                }
                return true;
            default:
                return false;
           
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityGasolineTanker == null || !_startPosX.HasValue || !_startPosY.HasValue) 
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush additonalBrush = new SolidBrush(EntityGasolineTanker.AdditionalColor);
        
        // бензобак
        if (EntityGasolineTanker.GasTank)
        {
            
            g.DrawEllipse(pen, _startPosX.Value + 7, _startPosY.Value + 19, 31, 31);
            g.DrawEllipse(pen, _startPosX.Value + 47, _startPosY.Value + 19, 31, 31);
            g.DrawRectangle(pen, _startPosX.Value + 20, _startPosY.Value + 20, 45, 30);
            
            g.FillEllipse(additonalBrush, _startPosX.Value + 7, _startPosY.Value+19, 31, 31);
            g.FillEllipse(additonalBrush, _startPosX.Value+ 47, _startPosY.Value + 19, 31, 31);
            g.FillRectangle(additonalBrush, _startPosX.Value + 20, _startPosY.Value + 21, 45, 30);

        }
        // сигнальный маяк
        if (EntityGasolineTanker.SignalBeacon)
        {
            g.DrawRectangle(pen, _startPosX.Value + 85, _startPosY.Value + 10, 7, 7);
            g.FillRectangle(additonalBrush, _startPosX.Value + 85, _startPosY.Value + 10, 7, 7);

        }

        // колеса
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillEllipse(brBlack, _startPosX.Value + 5, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(brBlack, _startPosX.Value + 25, _startPosY.Value + 50, 20, 20);
        g.FillEllipse(brBlack, _startPosX.Value + 85, _startPosY.Value + 50, 20, 20);

        // нижняя платформа
        Brush brBody = new SolidBrush(EntityGasolineTanker.BodyColor);
        g.FillRectangle(brBody, _startPosX.Value + 5, _startPosY.Value + 40, 100, 10);
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 40, 100, 10);
        
        
        // кабина
        Brush brLightBlue = new SolidBrush(Color.LightBlue);
        g.FillRectangle(brLightBlue, _startPosX.Value + 80, _startPosY.Value + 15, 25, 25);
        g.DrawRectangle(pen, _startPosX.Value + 80, _startPosY.Value + 15, 25, 25);
        
    }


}
